function vel_gradient, x, y, v, t, sigma, vgrad=vgrad, resid=resid, robust=robust
;+
; NAME:
;   VEL_GRADIENT
; PURPOSE:
;   To calculate the best-fit velocity gradient across a cloud.
;
; CALLING SEQUENCE:
;   posang = VEL_GRADIENT(x,y,v,t,sigma,vgrad=vgrad,resid=resid, [/ROBUST])
;
; INPUTS:
;   X, Y, V, T -- The X,Y,V and T_A components of a cloud
;   SIGMA -- RMS noise per channel
;
; KEYWORD PARAMETERS:
;   VGRAD -- magnitude of gradient in pixel units
;   RESID -- RMS residual in channel units
;   ROBUST -- keyword to perform robust instead of LSQ fit
;
; OUTPUTS:
;   The position angle CCW from the X=0 line of the coordinates used.
;
; MODIFICATION HISTORY:
;
;       Wed Jun 27 11:13:24 CDT 2012, Tony Wong
;		Written.
;
;-

;; CALCULATE THE FIRST MOMENT MAP AND ITS ERROR
  cubify, x, y, v, t, sz, cube = minicube, pad = 1
  vpix=indgen(sz[3])
  vmatrix=rebin(reform(vpix,1,1,sz[3]),sz[1],sz[2],sz[3])
  mom0 = total(minicube,3,/NAN)
  mom1 = total(vmatrix*minicube,3,/NAN)/mom0
  errmat = (vmatrix-rebin(mom1,sz[1],sz[2],sz[3]))^2
  mom1err = sigma/mom0 * sqrt(total(errmat,3))
  
;; FIT A PLANE TO THE MOMENT MAP (REQ FREUDENREICH ROUTINES)
  if (keyword_set(robust)) then begin  
    fitmap = ROB_MAPFIT( mom1,1,coef,resid )
;    print, 'COEFFICIENTS: ',coef
;    print, 'RMS RESIDUAL: ',resid
  endif else begin
    indvec = where(finite(mom1))
    xpix = indvec mod sz[1]
    ypix = (indvec mod (sz[1]*sz[2])) / sz[1]
    mom1vec = mom1[indvec]
    weights = 1/(mom1err^2)
    wtvec = weights[indvec]
    coef = PLANEFIT( xpix,ypix,mom1vec,wtvec,fitv )
;    print, 'COEFFICIENTS: ',coef
    resid = stddev( fitv - mom1vec )
;    print, 'RMS RESIDUAL: ',resid
  endelse

;; CALCULATE GRADIENT AND RETURN RESULTS
  vgrad=sqrt(coef[1]^2+coef[2]^2)   ; gradient in pixel units
  posang=atan(coef[2],coef[1])
  posang = posang+!pi*(posang lt 0)
;  print, 'GRADIENT, POS ANG: ',vgrad, posang
  return, posang
  
end
